@@ -1,6 +1,17 @@
 #' @rdname SSModel
 #' @export
-SSMcustom <- function(Z, T, R, Q, a1, P1, P1inf, index, n = 1) {
+#' @examples
+#' # add intercept to state equation by augmenting the state vector:
+#' # diffuse initialization for the intercept, gets estimated like other states:
+#' # for known fixed intercept, just set P1 = P1inf = 0 (default in SSMcustom).
+#' intercept <- 0
+#' model_int <- SSModel(Nile ~ SSMtrend(1, Q = 1469) +
+#' SSMcustom(Z = 0, T = 1, Q = 0, a1 = intercept, P1inf = 1), H = 15099)
+#'
+#' model_int$T
+#' model_int$T[1, 2, 1] <- 1 # add the intercept value to level
+#' out <- KFS(model_int)
+SSMcustom <- function(Z, T, R, Q, a1, P1, P1inf, index, n = 1, state_names = NULL) {
   if (missing(index))
     index <- 1
   p <- length(index)
@@ -66,11 +77,17 @@ SSMcustom <- function(Z, T, R, Q, a1, P1, P1inf, index, n = 1) {
       dim(P1inf) <- c(1, 1)
     } else {
       if (any(dim(P1inf)[1:2] != m))
-        stop("Misspecified P1inf, argument P1inf must be a (m x m) matrix, where m is the number of states..")
+        stop("Misspecified P1inf, argument P1inf must be a (m x m) matrix, where m is the number of states.")
     }
   }
   diag(P1inf)[diag(P1) > 0 || is.na(diag(P1))] <- 0
-  state_names <- paste0("custom", 1:m)
+  if (is.null(state_names)) {
+    state_names <- paste0("custom", 1:m)
+  } else {
+    if (length(state_names) != m) {
+      stop("Misspecified state_names, argument state_names must be a vector of length m, where m is the number of states.")
+    }
+  }
   list(index = index, m = m, k = k, p = p, n = n, Z = Z, T = T, R = R, Q = Q, a1 = a1,
     P1 = P1, P1inf = P1inf, tvz = dim(Z)[3] > 1, tvt = dim(T)[3] > 1, tvr = dim(R)[3] >
       1, tvq = dim(Q)[3] > 1, state_names = state_names)
